from __future__ import absolute_import, unicode_literals
from .base import Tree


def parse_tree(xml):
    """
    Parse a tree from an XML string.
    :param xml:
    :return:
    :rtype ooui.tree.Tree
    """
    from lxml import etree

    tree = etree.fromstring(xml)
    tree = tree.xpath('//tree')[0]
    return Tree(tree)


from lxml import etree


def preprocess_feature_tags(xml_str, feature_checker):
    doc = etree.fromstring(xml_str)

    for node in doc.xpath('//feature'):
        key = node.get('key')
        status = node.get('status', 'enabled')
        active = feature_checker(key)

        if status == 'enabled':
            condition_met = active
        elif status == 'disabled':
            condition_met = not active
        else:
            condition_met = True

        parent = node.getparent()
        if condition_met:
            for child in list(node):
                parent.insert(parent.index(node), child)
            parent.remove(node)
        else:
            parent.remove(node)

    return etree.tostring(doc, encoding='unicode')



from __future__ import absolute_import, unicode_literals
from ooui.helpers.conditions import ConditionParser


class Tree(object):
    def __init__(self, element):
        """
        :param element: lxml.etree._Element
        """
        self._string = element.get('string')
        self._infinite = element.get('infinite', None)
        self._colors = element.get('colors', None)
        self._status = element.get('status', None)
        self._editable = element.get('editable', None)
        self._element = element


    @property
    def string(self):
        return self._string

    @property
    def infinite(self):
        return self._infinite

    @property
    def fields(self):
        res = []
        for field in self._element.xpath('field'):
            res.append(field)
        return res

    @property
    def fields_in_conditions(self):
        res = {}
        if self._colors:
            parser = ConditionParser(self._colors)
            res['colors'] = list(parser.involved_fields)
        if self._status:
            parser = ConditionParser(self._status)
            res['status'] = list(parser.involved_fields)
        return res



from __future__ import absolute_import, unicode_literals


class Aggregator:
    def __init__(self, data, field_definitions, precisions=None):
        self.data = data
        self.field_definitions = field_definitions
        self.precisions = precisions or {}

    def process(self):
        results = {}
        for field, functions in self.field_definitions.items():
            precision = self.precisions.get(field)
            values = [item[field] for item in self.data if field in item]
            results[field] = {}
            if 'sum' in functions:
                result = sum(values)
                results[field]['sum'] = precision and round(result, precision) or result
            if 'count' in functions:
                results[field]['count'] = round(len(values), precision)
            if 'avg' in functions:
                result = sum(values) / float(len(values)) if values else 0
                results[field]['avg'] = precision and round(result, precision) or result
            if 'max' in functions:
                result = max(values) if values else 0
                results[field]['max'] = precision and round(result, precision) or result
            if 'min' in functions:
                result = min(values) if values else 0
                results[field]['min'] = precision and round(result, precision) or result
        return results
